* Q-Config list selection routine
*  Rich Mellor
* v1.00 16/5/00

* NOTES
* -----
* Max length of the menu name is 12 characters - any more disrupts display
* May want to add ability to edit items in menu - see line 220
* Need one functions: C_SEL$(no [,edit] )
* if pass one parameter only - it returns the status list from the given
* select block - X for unavailable, 1 for selected, 0 for available (already implemented)
* if pass two parameters, it returns the value of the specified item number in
* the given select block.

* At present, if this is the only configuration block, there does not seem anyway
* of notifying MenuConfig that the item has been changed and may need to be saved.

* Some keys used by QMenu structure
thp..ptr equ       15           ; bit set for pointer parameter
thp..cal equ       14           ; bit set for call parameter
thp..ret equ       13           ; bit set for return paramter
thp..opt equ       12           ; bit set if parameter is optional
thp..nnl equ       11           ; bit set if negative for null - NOT thp..ptr
thp..arr equ       11           ; bit set for array - thp..ptr

thp..sgn equ        1           ; bit set if value is signed
thp..chr equ        2           ; bit set if character allowed
thp..byt equ        3           ; bit set if byte value allowed/required
thp..wrd equ        4           ; bit set if word value allowed/required
thp..lng equ        5           ; bit set if long value allowed/required
thp..cid equ        6           ; bit set for channel ID
thp..fp8 equ        7           ; bit set for eight byte floating point

thp..str equ        8           ; standard string
thp..sst equ        9           ; sub-string

thp..undln equ      2           ; set to add an underline character before options

thp.char equ    $0004           ; character
thp.ubyt equ    $0008           ; unsigned byte
thp.sbyt equ    $000a           ; signed byte
thp.uwrd equ    $0010           ; unsigned word
thp.swrd equ    $0012           ; signed word
thp.ulng equ    $0020           ; unsigned long
thp.slng equ    $0022           ; signed long
thp.chid equ    $0040           ; channel ID
thp.fp8  equ    $0082           ; eight byte floating point
thp.str  equ    $0100           ; string
thp.sstr equ    $0200           ; sub-string
thp.nnul equ    1<<thp..nnl     ; negative null (-1)
thp.arr  equ    1<<thp..arr     ; array
thp.opt  equ    1<<thp..opt     ; optional
thp.upd  equ    1<<thp..ptr+1<<thp..cal+1<<thp..ret ; updated parameter
thp.call equ    1<<thp..ptr+1<<thp..cal             ; call parameter
thp.ret  equ    1<<thp..ptr+1<<thp..ret             ; return parameter
thp.ptr  equ    1<<thp..ptr                         ; call or return parameter
thp.undln equ   1<<thp..undln                       ; How options to be underlined

ls_mennm equ    $04             ; pointer to menu name
ls_itlty equ    $08             ; item list type
ls_itlst equ    $0c             ; item list
ls_stlty equ    $10             ; status list type
ls_stlst equ    $14             ; item status list
ls_mictl equ    $18             ; menu item control
ls_lines equ    $1c             ; max number of lines
ls_colms equ    $20             ; max number of columns
ls_xpos  equ    $24             ; x position of left hand side
ls_ypos  equ    $28             ; y position of top
ls_mainc equ    $2c             ; main colourway
ls_listc equ    $30             ; list window colourways
ls_item  equ    $38             ; pointer to return item number

* CONFIG BLOCK DETAILS - LEVEL 1

        dc.b   '<<QCFX>>'   ; Config Block Identifier
        dc.b   '01'         ; Config Level Indicator
        dc.w   7            ; Software Name
        dc.b   'QConfig'
        dc.b   0            ; Word Alignment
        dc.w   4            ; Version Number
        dc.b   '1.00'
* TABLE OF ITEMS
        dc.b   6            ; Type of Item #1 (SELECTION)
        dc.b   'A'          ; Item Selection Keystroke
        dc.w   CFG000-*     ; Pointer To Item
        dc.w   LIST_THG-*   ; Pointer To Item Pre-Processing Routine
        dc.w   0            ; Pointer To Item Post-Processing Routine
        dc.w   CFG001-*     ; Pointer To Description of Item
        dc.w   CFG002-*     ; Pointer To Attributes of Item

        dc.w   -1           ; END OF ITEMS TABLE

* ITEM DETAILS
CFG001  dc.w   16           ; Description of Item #1
        dc.b   'Select Test Menu'
CFG002  dc.b   0            ; Unused
        dc.b   '0'          ; Selection Keystroke for Selection #
        dc.w   6            ; Description of Selection
        dc.b   'Test A'
        dc.b   0            ; Unused
        dc.b   '1'          ; Selection Keystroke for Selection #
        dc.w   6            ; Description of Selection
        dc.b   'Test B'
        dc.b   0            ; Unused
        dc.b   '2'          ; Selection Keystroke for Selection #
        dc.w   6            ; Description of Selection
        dc.b   'Test C'
        dc.b   0            ; Unused
        dc.b   '3'          ; Selection Keystroke for Selection #
        dc.w   6            ; Description of Selection
        dc.b   'Test D'
        dc.w   -1           ; End of Selection List

* Status Table for Item #1
* 128=Selected, 0=available, 16=unavailable

CFG000
CNF001  dc.b   0            ; Status for Selection #1
        dc.b   16           ; Status for Selection #2
        dc.b   128          ; Status for Selection #3
        dc.b   0            ; Status for Selection #4
* End of STATUS TABLE

* This is a generic pre-processing routine (c) Rich Mellor 2000
* Which enables the user to control toggle of several items from one menu.

* Now our code to provide a multi-item select menu.
* First of all find the Special Thing (THG%)

LIST_THG

* First of all, we need to rearrange the selection items and attributes to
* suit QMenu

* On Entry - A0 points to the item (entries for menu)
* A2 points to the attributes (status for each item)
* A1 points to the menu description
* A3 points to a 4K buffer

     lea      temp_menu,a5
     move.l   a1,(a5)+             ; Store pointer to menu description
     move.l   a3,(a5)+             ; Store pointer to 4K buffer
     move.l   a2,(a5)+             ; Store pointer to attributes (text for entries)
     move.l   a0,(a5)              ; Store pointer to item (status)

     move.w   (a1),d1              ; Check length of menu name
     cmpi.w   #11,d1
     bls.s    name_ok              ; Ensure that it is a maximum of 11 characters
     move.w   #11,(a1)             ; Any more will corrupt the menu name

name_ok
     moveq    #0,d0                ; Reset number of menu entries
     moveq    #0,d1                ; Reset temp variable
     moveq    #0,d2                ; Reset max string length

     move.l   a2,-(a7)             ; Store pointer to entries

ck_str
     move.w   (a2)+,d1             ; Look at selection key/-1 for end of list
     cmpi.w   #-1,d1
     beq.s    copy_strings
     move.w   (a2)+,d1             ; Get length of string
     cmp.w    d1,d2                ; If longer than string length - update
     bcc.s    short_str
     move.w   d1,d2
short_str
     btst     #0,d1                ; Odd length string?
     beq.s    nxt_str
     add.w    #1,d1                ; Boundary alignment
nxt_str
     adda.l   d1,a2                ; Move to next entry
     addq.w   #1,d0                ; Increment number of strings
     bra.s    ck_str

copy_strings
     move.l   (a7)+,a2
     tst.w    d0
     beq      no_thing             ; Exit if no entries in menu!
     subq.w   #1,d0                ; Stored as number of entries -1
     lea      string_no,a3
     move.w   d0,(a3)              ; Store number of menu items
     move.w   d2,4(a3)             ; Store max length of string
     addq.w   #2,d2                ; Calculate multiplier
     move.w   d2,2(a3)             ; Store index multiplier

     lea      status_no,a3
     move.w   d0,(a3)              ; Store number of status flags
     lea      array_string,a3

nxt_cstr
     moveq    #0,d1
     move.w   (a2)+,d1             ; Look at selection key/-1 for end of list
     cmpi.w   #-1,d1               ; Exit on last entry
     beq.s    copy_status
     move.w   (a2)+,d1             ; Get length of string
     move.l   a3,a4
     adda.l   d2,a4                ; Store offset of next string in array
     move.w   d1,(a3)+             ; Store this
     tst.w    d1
     beq.s    nxt_cstr1            ; Ignore null strings
     btst     #0,d1                ; Is it an odd string?
     bne.s    cop_str              ; Make it into even length
     subq.w   #1,d1
cop_str
     move.b   (a2)+,(a3)+          ; Copy across the string as required
     dbf      d1,cop_str
nxt_cstr1
     move.l   a4,a3                ; Get start of next string
     bra.s    nxt_cstr

copy_status
     lea      array_status,a3
     moveq    #0,d1
cop_stat
     move.b   (a0)+,d1             ; Get status flags - convert to words

* To make the entries editable, add following line and copy text back as well
* as statuses - length of text may be a problem... - edited text seems to be
* two bytes shorter.  Would also need to ensure that when copy back, strings
* are same length as original so as not to destroy the config block.
* bset #14,d1

     move.w   d1,(a3)+             ; and store as an integer array for QMenu
     dbf      d0,cop_stat

* Now operate the LIST thing from QMenu

     moveq    #0,d0
     trap     #1                   ; Get system info
     moveq    #-7,d0
     move.l   $b8(a0),d5           ; Get start of Thing List
     beq.s    no_thing             ; Thing system not found!!!
F_LOOP
     move.l   d5,a0                ; Pointer to next Thing Linkage Block
     move.l   (a0),d5              ; Are we at end of list?
     bne.s    F_LOOP
     move.l   $10(a0),a3           ; A3 points to start of Thing
     cmpi.l   #'THG%',(a3)         ; Is it the special thing??
     bne.s    no_thing
     cmpi.l   #-1,4(a3)            ; Are we certain?
     bne.s    no_thing
     move.l   8(a3),a4             ; A4 points to THH_ENTR routine
     bsr      setup_params         ; Setup the parameters for the thing
     move.l   a3,-(a7)
     bsr      bt_usmen             ; Use the specified Thing
     tst.l    d0
     bne.s    no_QMenu
     lea.l    $18(a1),a4
     lea      params,a1
     jsr      (a4)                 ; Call the extension code itself
     tst.l    d0
     bne.s    no_Qmenu
     move.l   (a7)+,a3
     tst.l    d0
     bne.s    no_thing             ; Oops - an error!!

* Close down routine
    lea       TH_NAME,a0           ; Free thing
    moveq     #$29,d0
    moveq     #-1,d1               ; This job
    moveq     #0,d2
    moveq     #-1,d3               ; Timeout
    move.l    8(a3),a4             ; Point to THH_ENTR and call
    jsr       (a4)

* Need to do something with the code - copy back across the status block to the
* config table to be read by user.

    tst.l     d0
    bne.s     no_thing             ; An error has occured

* Copy status table back before exiting pre-processing routine

    lea       temp_menu,a5
    move.l    3*4(a5),a2           ; Grab pointer to status flags
    lea       status_no,a5
    move.w    (a5),d0              ; Grab number of statuses to copy across (-1)
    lea       array_status,a5
ret_stat
    move.w    (a5)+,d1             ; Copy across each status - convert to byte
    move.b    d1,(a2)+
    dbf       d0,ret_stat

    moveq     #1,d0                ; Signal that no more work needed on this list
no_thing
    bra.s     return_thing
    rts

no_QMenu
    move.l    (a7)+,a3
    moveq     #0,d0                ; QMenu not present - use standard Config
return_thing
    lea       temp_menu,a5
    move.l    (a5)+,a1             ; Restore pointer to menu description
    move.l    (a5)+,a3             ; Restore pointer to 4K buffer
    move.l    (a5)+,a2             ; Restore pointer to attributes (text for entries)
    move.l    (a5),a0              ; Restore pointer to item (status)
    rts

TH_NAME
    dc.w      5
    dc.b      'Menus',0            ; Name of thing to use

bt_usmen
    lea       TH_NAME,a0
    move.l    #'LIST',d2           ; Name of Extension Thing to use
    moveq     #-1,d1               ; Current Job ID
    moveq     #-1,d3               ; Timeout
    moveq     #$28,d0              ; Call sms.uthg without TRAP #1
    jsr       (a4)
    rts

setup_params

* We first of all need to set up the list of items to be selected and their
* statuses as standard QDOS arrays.

    lea.l     item_list,a2         ; Convert item list to a pointer
    lea.l     array_string,a1
    move.l    a1,(a2)
    lea.l     item_status_list,a2  ; Convert item status list to a pointer
    lea.l     array_status,a1
    move.l    a1,(a2)

* Now we can set up the parameters to call QMenu LIST menu with
    lea       params,a1

    move.l    #(thp.call+thp.str)<<16,ls_mennm-4(a1)        ; control longword
    lea       temp_menu,a2
    move.l    (a2),a2                                       ; Grab pointer to menu name
    move.l    a2,ls_mennm(a1)                               ; Menu name pointer

    move.w    #thp.call+thp.arr+thp.str,d2                  ; item list type
    swap      d2
    move.w    #0,d2
    move.l    d2,ls_itlty(a1)                               ; Absolute offset
    lea       item_list,a2                                  ; Pointer to item list
    move.l    a2,ls_itlst(a1)

    move.w    #thp.upd+thp.arr+thp.uwrd,d2                  ; item status list type
    swap      d2
    move.w    #0,d2
    move.l    d2,ls_stlty(a1)                               ; Absolute offset
    lea       item_status_list,a2                           ; Pointer to item status list
    move.l    a2,ls_stlst(a1)

    move.l    #thp.undln,ls_mictl(a1)                       ; How items displayed

    move.l    #0,ls_lines(a1)                               ; Max no. of lines

    move.l    #0,ls_colms(a1)                               ; Max no. of columns

    move.l    #15,ls_xpos(a1)                               ; x position

    move.l    #100,ls_ypos(a1)                              ; y position

    move.l    #-1,ls_mainc(a1)                              ; Main colourway

    move.l    #-1,ls_listc(a1)                              ; List colourway

    move.l    #(thp.ret+thp.uwrd)<<16,ls_item-4(a1)
    lea       $40(a1),a2
    move.w    #0,(a2)                                       ; insert item at start
    move.l    a2,ls_item(a1)                                ; pointer to return parameter
    rts

* Parameters now follow for the LIST extension thing:

params
    ds.b      $44

* Following is to be the list of items for selection - they will need to be
* copied across from the config block to give a list as a 2 dimensional
* string array

item_list
           dc.l         0               ; Offset of array values from base of
*                                       ; variable value area - to be filled in
           dc.w         2               ; Number of dimensions
string_no  dc.w         0               ; Maximum Number of strings -1
           dc.w         8               ; Index multiplier
           dc.w         6               ; Max length of strings
           dc.w         1               ; Index multiplier

* Following to be copied across from the config block
* Also work out number of entries (less one) and fill in at String_no (word)
* (number of entries less 1)

Array_string
           ds.b         2*1024

item_status_list
* Note that the first item must NOT be unavailable.

           dc.l         0               ; Offset of array values from base of
*                                       ; variable value area - to be filled in
           dc.w         1               ; Number of dimensions
status_no  dc.w         0               ; One entry per string -1 (to be filled in)
           dc.w         2               ; Index multiplier

* Following to be copied across from the config block
* Also work out number of entries (less one) and fill in at Status_no (word)
* (number of entries less 1)

Array_status
           ds.b         1024

temp_menu  dc.l         0               ; Room for pointer to menu name
           dc.l         0               ; Room for pointer to 4K buffer
           dc.l         0               ; Room for attributes pointer
           dc.l         0               ; Room for item pointer

        END
